#include "stdafx.h"
#include "SkyTune.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

class CSkyTuneApp : public CWinApp
{
protected:
  virtual BOOL InitInstance();

  void DoTheWork();
};


//The one and only CWinApp instance
CSkyTuneApp theApp;

BOOL CSkyTuneApp::InitInstance()
{
  //Initialize COM
  CoInitialize(NULL);

  //Call the helper function which does the real work
  DoTheWork();

  //Terminate COM
  CoUninitialize();

  return FALSE;
}

void CSkyTuneApp::DoTheWork()
{
  //By default use an ini file from the directory where the exe is located
  TCHAR pszIni[_MAX_PATH];
  GetModuleFileName(NULL, pszIni, _MAX_PATH);
  TCHAR pszDrive[_MAX_DRIVE];
  TCHAR pszDir[_MAX_DIR];
  TCHAR pszFname[_MAX_FNAME];
  _tsplitpath(pszIni, pszDrive, pszDir, pszFname, NULL);
  _tmakepath(pszIni, pszDrive, pszDir, pszFname, _T("ini"));
  free((void*)m_pszProfileName);
  m_pszProfileName=_tcsdup(pszIni);

  //Parse the command line
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
  CString sButtons(cmdInfo.m_strFileName);
  if (sButtons.IsEmpty())
  {
    AfxMessageBox(_T("Usage: SkyTune ButtonsToSend"));
    return;
  }

  //Check to see if we should use a RedRat3
  CString sRedRat3Name = GetProfileString(_T("General"), _T("RedRat3Name"), _T(""));
  if (sRedRat3Name.GetLength())
  {
    //Create the RedRatLoopkup object
    RedRat::IRedRatLookupPtr RedRatLookup;
    HRESULT hr = RedRatLookup.CreateInstance(__uuidof(RedRat::RedRatLookup));
    if (FAILED(hr))
    {
      CString sError;
      sError.Format(_T("Failed to create RedRat3 Connections COM object, Error:%x"), hr);
      AfxMessageBox(sError);
      return;
    }

    try
    {
      //Get the RedRat device we want
      RedRat::ICCWRedRat3Ptr RedRat = RedRatLookup->GetRedRat3(_bstr_t(sRedRat3Name));

      //Get the name of the XML file which we use for the signal database
      CString sSignalDatabase = GetProfileString(_T("General"), _T("SignalDatabase"), _T(""));
      if (sSignalDatabase.IsEmpty())
      {
        CString sError;
        sError.Format(_T("Please specify a \"SignalDatabase\" value in the ini file \"%s\""), m_pszProfileName);
        AfxMessageBox(sError);
        return;
      }

      //Get the name of the Device Name which we pass to OutputSignal method (This is a limitation of SkyTune, you must use a Signal Database where all the codes or under the one device name)
      CString sDeviceName = GetProfileString(_T("General"), _T("DeviceName"), _T(""));
      if (sDeviceName.IsEmpty())
      {
        CString sError;
        sError.Format(_T("Please specify a \"DeviceName\" value in the ini file \"%s\""), m_pszProfileName);
        AfxMessageBox(sError);
        return;
      }

      //Create the signal database object
      RedRat::ISignalDBPtr SignalDB;
      hr = SignalDB.CreateInstance(__uuidof(RedRat::CCWSignalDB));
      if (FAILED(hr))
      {
        CString sError;
        sError.Format(_T("Failed to create RedRat3 Signal Database COM object, Error:%x"), hr);
        AfxMessageBox(sError);
        return;
      }

      //Load up the signal database
      SignalDB->AddDeviceDB(_bstr_t(sSignalDatabase));

      //Let the device known about the signal database
      RedRat->SetSignalDB(SignalDB);

      //Parse each button sequence from the command line passed to us
      while (sButtons.GetLength())
      {
        int nSeparator = sButtons.Find(_T(','));
        CString sThisButton;
        if (nSeparator != -1)
        {
          sThisButton = sButtons.Left(nSeparator);
          sButtons = sButtons.Right(sButtons.GetLength() - nSeparator - 1);
        }
        else
        {
          sThisButton = sButtons;
          sButtons.Empty();
        }
        sThisButton.TrimLeft();
        sThisButton.TrimRight();

        //Look up the button in the ini file
        if (sThisButton == _T("DELAY"))
        {
          Sleep(100);
        }
        else
        {
          //Send the button
          RedRat->OutputSignal(_bstr_t(sDeviceName), _bstr_t(sThisButton));
        }
      }
    }
    catch(_com_error& e)
    {
      //Report the error
      CString sError;
      sError.Format(_T("RedRat3 COM Exception, Error:%x, %s"), e.Error(), CString(e.Description().operator wchar_t*()));
      AfxMessageBox(sError);
    }    
  }
  else
  {
    //See if we need to open a new COM object or connect to an existing one
    REDRATLib::IRedRatConnectionsPtr connections;
    HRESULT hr = connections.CreateInstance(__uuidof(REDRATLib::RedRatConnections));
    if (FAILED(hr))
    {
      CString sError;
      sError.Format(_T("Failed to create RedRat2 Connections COM object, Error:%x"), hr);
      AfxMessageBox(sError);
      return;
    }

    try
    {
      //Determine the port on which the redrat is located
      int nPort = GetProfileInt(_T("General"), _T("Port"), 1);

      //See is our port already open
      long nConnections = connections->Count;
      BOOL bFoundOurPort = FALSE;
      REDRATLib::IRedRatConnectionPtr connection;
      for (long i=0; i<nConnections && !bFoundOurPort; i++)
      {
        connection = connections->Item(i);
        if (connection->Port == nPort)
          bFoundOurPort = TRUE;
      }

      //If our port was not found, then open it
      if (!bFoundOurPort)
      {
        REDRATLib::IRedRatPortPtr port;
        HRESULT hr = port.CreateInstance(__uuidof(REDRATLib::RedRatPort));
        if (FAILED(hr))
        {
          CString sError;
          sError.Format(_T("Failed to create RedRat2 port COM object, Error:%x"), hr);
          AfxMessageBox(sError);
          return;
        }
        connection = port->CreateConnection(nPort, VARIANT_TRUE);
      }

      //Parse each button sequence from the command line passed to us
      while (sButtons.GetLength())
      {
        int nSeparator = sButtons.Find(_T(','));
        CString sThisButton;
        if (nSeparator != -1)
        {
          sThisButton = sButtons.Left(nSeparator);
          sButtons = sButtons.Right(sButtons.GetLength() - nSeparator - 1);
        }
        else
        {
          sThisButton = sButtons;
          sButtons.Empty();
        }
        sThisButton.TrimLeft();
        sThisButton.TrimRight();

        //Look up the button in the ini file
        if (sThisButton == _T("DELAY"))
        {
          Sleep(100);
        }
        else
        {
          CString sCommand = GetProfileString(_T("Buttons"), sThisButton);
          if (sCommand.GetLength())
          {
            //Send the button
            connection->Send(_bstr_t(sCommand));
          }
          else
          {
            CString sError;
            sError.Format(_T("Failed to find the button definition %s in the ini file \"%s\""), sThisButton, m_pszProfileName);
            AfxMessageBox(sError);
            return;
          }
        }
      }
    }
    catch(_com_error& e)
    {
      //Report the error
      CString sError;
      sError.Format(_T("RedRat2 COM Exception, Error:%x, %s"), e.Error(), CString(e.Description().operator wchar_t*()));
      AfxMessageBox(sError);
    }
  }
}

